@@ -76,9 +76,6 @@ static char *buildopts[] =
 #ifdef SPOTIFY_LIBRESPOTC
     "librespot-c",
 #endif
-#ifdef SPOTIFY_LIBSPOTIFY
-    "libspotify",
-#endif
 #ifdef LASTFM
     "LastFM",
 #else
@@ -258,7 +255,7 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
   char strport[8];
   int yes = 1;
   int no = 0;
-  int fd;
+  int fd = -1;
   int ret;
 
   cfgaddr = cfg_getstr(cfg_getsec(cfg, "general"), "bind_address");
@@ -275,7 +272,7 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
       return -1;
     }
 
-  for (ptr = servinfo, fd = -1; ptr != NULL; ptr = ptr->ai_next)
+  for (ptr = servinfo; ptr != NULL; ptr = ptr->ai_next)
     {
       if (fd >= 0)
 	close(fd);
@@ -338,7 +335,8 @@ net_bind(short unsigned *port, int type, const char *log_service_name)
   return fd;
 
  error:
-  close(fd);
+  if (fd >= 0)
+    close(fd);
   return -1;
 }
 
@@ -398,7 +396,11 @@ safe_atoi32(const char *str, int32_t *val)
   char *end;
   long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atoi32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtol(str, &end, 10);
@@ -407,21 +409,18 @@ safe_atoi32(const char *str, int32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > INT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -436,7 +435,11 @@ safe_atou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atou32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoul(str, &end, 10);
@@ -445,21 +448,18 @@ safe_atou32(const char *str, uint32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -474,7 +474,11 @@ safe_hextou32(const char *str, uint32_t *val)
   char *end;
   unsigned long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_hextou32 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoul(str, &end, 16);
@@ -483,21 +487,18 @@ safe_hextou32(const char *str, uint32_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT32_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -512,7 +513,11 @@ safe_atoi64(const char *str, int64_t *val)
   char *end;
   long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atoi64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoll(str, &end, 10);
@@ -521,21 +526,18 @@ safe_atoi64(const char *str, int64_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > INT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -550,7 +552,11 @@ safe_atou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_atou64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoull(str, &end, 10);
@@ -573,7 +579,6 @@ safe_atou64(const char *str, uint64_t *val)
   if (intval > UINT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -588,7 +593,11 @@ safe_hextou64(const char *str, uint64_t *val)
   char *end;
   unsigned long long intval;
 
-  *val = 0;
+  if (str == NULL)
+    {
+      DPRINTF(E_SPAM, L_MISC, "Input to safe_hextou64 is NULL\n");
+      return -1;
+    }
 
   errno = 0;
   intval = strtoull(str, &end, 16);
@@ -597,21 +606,18 @@ safe_hextou64(const char *str, uint64_t *val)
       || ((errno != 0) && (intval == 0)))
     {
       DPRINTF(E_DBG, L_MISC, "Invalid integer in string (%s): %s\n", str, strerror(errno));
-
       return -1;
     }
 
   if (end == str)
     {
       DPRINTF(E_DBG, L_MISC, "No integer found in string (%s)\n", str);
-
       return -1;
     }
 
   if (intval > UINT64_MAX)
     {
       DPRINTF(E_DBG, L_MISC, "Integer value too large (%s)\n", str);
-
       return -1;
     }
 
@@ -705,13 +711,15 @@ safe_snreplace(char *s, size_t sz, const char *pattern, const char *replacement)
       if (dst + num > s + sz)
 	return -1; // Not enough room
 
-      // Shift everything after the pattern to the right, use memmove since
-      // there might be an overlap
+      // Move everything after the pattern, use memmove since there might be an overlap
       memmove(dst, src, num);
 
       // Write replacement, no null terminater
       memcpy(ptr, replacement, r_len);
 
+      // String now has a new length
+      s_len += r_len - p_len;
+
       // Advance ptr to avoid infinite looping
       ptr = dst;
     }
