@@ -1506,7 +1506,7 @@ device_streaming_cb(struct output_device *device, enum output_device_state statu
   // We don't do this in the other cb's because they are triggered by a command
   // and thus the update should be done as part of the command completion (which
   // can better determine which type of listener event to use)
-  status_update(player_state, LISTENER_SPEAKER);
+  status_update(player_state, LISTENER_SPEAKER | LISTENER_VOLUME);
 }
 
 static void
@@ -3631,7 +3631,13 @@ player_init(void)
   int ret;
 
   speaker_autoselect = cfg_getbool(cfg_getsec(cfg, "general"), "speaker_autoselect");
-  clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+  clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "library"), "clear_queue_on_stop_disable");
+
+  /* Handle deprecated config options, note that this is also in library.c */
+  if (0 < cfg_opt_size(cfg_getopt(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable")))
+    {
+      clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
+    }
 
   CHECK_NULL(L_PLAYER, player_settings_category = settings_category_get("player"));
   ret = SETTINGS_GETINT(player_settings_category, PLAYER_SETTINGS_MODE_REPEAT);
